/**
  *  \file game/config.h
  *
  *  (c) 2001-2005 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CONFIG_H
#define PCC_V2_CONFIG_H

#include "cpluslib/string.h"
#include "cpluslib/types.h"
#include "io/io.h"
#include "io/streams.h"

/* configuration structure */
#include "game/config-s.h"

/** Configuration Parser Information. This structure defines one
    config option. */
struct TPConfigConvert {
    const char*      name;      ///< Name of this option, in lowercase
    TStructureMember type;      ///< Type in TGameConfig structure
    std::size_t      offset;    ///< Offset in TGameConfig structure
    unsigned short   count;     ///< Number of array elements
    bool             arrayized; ///< true if arrayized
    int32            min, max;  ///< Limits
    const char*      deflt;     ///< default value, as string
    TStructureMember t_type;    ///< Type in HCONFIG.HST
    unsigned short   t_count;   ///< Number of items in HCONFIG.HST
    unsigned short   t_ofs;     ///< Offset in HCONFIG.HST
};

struct TPConfigAliasTable {
    const char*      name;
    unsigned         index;
};

extern TGameConfig config;

void loadPConfig(Ptr<Stream> pconf);
void loadHConfig(Ptr<Stream> hconf);
void loadDefaultConfig();

void assignConfig(int index, string_t config_value);

int lookupConfigOption(string_t name);
int lookupConfigOption(const char* name);

void initConfig();

inline int raceId(int player) {
    if (player > 0 && player <= 11)
        return config.PlayerRace[player-1];
    else
        return player;
};

inline int raceMissionId(int player) {
    if (player > 0 && player <= 11)
        return config.PlayerSpecialMission[player-1];
    else
        return player;
};

inline int playerIndex(int player) {
    if (player > 0 && player <= 11)
        return player-1;
    else
        return 0;
}

#define CONFIG_ARRAY(name,player) (::config.name[::playerIndex(player)])

#endif
