/**
  *  \file game/lock.h
  *  \brief Locking onto Objects
  *
  *  This module provides functions for locking at objects (mouse click
  *  into starchart). In addition, it provides a means for configuring
  *  what mouse button locks onto what.
  *
  *  (c) 2001-2005 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_LOCK_H
#define PCC_V2_GAME_LOCK_H

#include "game/coord.h"

class GUniverse;

/** Lock mode. Identifies a set of object types associated with a
    particular event. */
enum TLockMode {
    lm_Left,                    ///< Left mouse button.
    lm_Right,                   ///< Right mouse button.
    lm_MAX
};

/** Lock item. A bitfield of these defines a set of object types
    associated with a particular event. */
enum TLockItems {
    li_Planet    = 1,
    li_Ship      = 2,
    li_Ufo       = 4,
    li_Marker    = 8,
    li_Minefield = 16
};

class GLockData {
    GPoint clicked;
    GPoint min, max;
    GPoint found;
    bool   marked;
    long   min_dist;
 public:
    GLockData(GPoint clicked, bool marked);
    GLockData(GPoint clicked, GPoint min, GPoint max, bool marked);
    void addPoint(GPoint pt, bool marked);

    /** Get found point. If the found object is across a map border,
        this will return the coordinates mapped into the map instance
        of /clicked/. */
    GPoint getPoint() const
        { return found; }
};

void findPlanet(const GUniverse& univ, GLockData& data);
void findShip(const GUniverse& univ, GLockData& data);
void findUfo(const GUniverse& univ, GLockData& data);
void findMarker(const GUniverse& univ, GLockData& data);
void findMinefield(const GUniverse& univ, GLockData& data);

void setLockMode(TLockMode mode, int items);
int  getLockMode(TLockMode mode);
void findItem(const GUniverse& univ, GLockData& data, TLockMode mode);

#endif
