/**
  *  \file game/player.h
  *
  *  (c) 2001-2005 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_PLAYER_H
#define PCC_V2_PLAYER_H

#include <string>
#include "cpluslib/smallset.h"
#include "cpluslib/string.h"
#include "io/streams.h"

/** Player-to-Player Relation. */
enum GPlayerRelation {
    is_Me,                      ///< It's we.
    is_Ally,                    ///< It's an ally.
    is_Enemy                    ///< It's an enemy.
};

/** Set of players. */
typedef SmallSet<int> GPlayerSet;

/** Race name list. Routines to parse and query a race.nm file. */
class GRaceNameList {
    string_t short_names[13], long_names[13], adjectives[13];
 public:
    GRaceNameList(Stream* p = 0);
    ~GRaceNameList();
    void             load(Stream&);
    const string_t&  getShortName(int player);
    const string_t&  getFullName(int player);
    const string_t&  getAdjName(int player);

    string_t         expandNames(string_t text);
};

extern GRaceNameList  racename_data;
extern const char     player_chars[];

inline string_t
getShortName(int player)
{
    return racename_data.getShortName(player);
}

inline string_t
getFullName(int player)
{
    return racename_data.getFullName(player);
}

inline string_t
getAdjName(int player)
{
    return racename_data.getAdjName(player);
}

inline string_t
expandRaceNames(const string_t& text)
{
    return racename_data.expandNames(text);
}

/* These player ids are used for display, not for rules */
int              getPlayerId();
int              getRealPlayerId();
void             setPlayerIds(int eff, int real);
GPlayerRelation  getPlayerRelation(int n);
int              getPlayerIdFromChar(char c);
bool             allowRegistered(int player);
void             initRaceNames();

inline char getPlayerChar(int player)
{
    return player_chars[player];
}


#endif
