/**
  *  \file game/sim-data.cc
  */

#include "game/sim.h"

GSimObject::GSimObject()
    : id(0), damage(0), shield(100), owner(12), experience_level(0), flags(0)
{
    storeBasicString(name, "?");
    storeBasicString(fcode, "???");
}

GSimObject::~GSimObject()
{ }

void
GSimObject::setChanged()
{ }

int
GSimObject::getId() const
{
    return id;
}

void
GSimObject::setId(int id)
{
    this->id = id;
    setChanged();
}

string_t
GSimObject::getName() const
{
    return getBasicString(name);
}

void
GSimObject::setName(string_t name)
{
    storeBasicString(this->name, name);
    setChanged();
}

string_t
GSimObject::getFCode() const
{
    return getBasicString(fcode);
}

void
GSimObject::setFCode(string_t fcode)
{
    storeBasicString(this->fcode, fcode);
    setChanged();
}


int
GSimObject::getDamage() const
{
    return damage;
}

void
GSimObject::setDamage(int damage)
{
    this->damage = damage;
    setChanged();
}

int
GSimObject::getShield() const
{
    return shield;
}

void
GSimObject::setShield(int shield)
{
    this->shield = shield;
    setChanged();
}

int
GSimObject::getOwner() const
{
    return owner;
}

void
GSimObject::setOwner(int owner)
{
    this->owner = owner;
    setChanged();
}

int
GSimObject::getExperienceLevel() const
{
    return experience_level;
}

void
GSimObject::setExperienceLevel(int experience_level)
{
    this->experience_level = experience_level;
    setChanged();
}

int
GSimObject::getFlags() const
{
    return flags;
}

void
GSimObject::setFlags(int flags)
{
    this->flags = flags;
    setChanged();
}

/** Get this object as a ship.
    \return \c this if this is a ship, 0 otherwise. */
GSimShip*
GSimObject::asShip()
{
    return 0;
}

/** Get this object as a planet.
    \return \c this if this is a planet, 0 otherwise. */
GSimPlanet*
GSimObject::asPlanet()
{
    return 0;
}

/******************************** GSimShip *******************************/

/** Construct blank simulator ship. */
GSimShip::GSimShip()
    : crew(10), hull(0), mass(100), beam_type(0), beam_count(0),
      torp_type(0), torp_lcount(0), bay_count(0), ammo(0),
      engine_type(NUM_ENGINES), aggressiveness(agg_Passive)
{ }

/** Construct from file data.
    \param file_data [in] Ship record read from a .ccb file */
GSimShip::GSimShip(const TSimShipData& file_data, int version)
    : crew(file_data.vcro.crew), hull(file_data.hull_type),
      mass(file_data.mass), beam_type(file_data.vcro.beam_type),
      beam_count(file_data.vcro.beam_count), torp_type(file_data.vcro.torp_type),
      torp_lcount(file_data.vcro._torp_launcher_count),
      bay_count(file_data.vcro.bay_count), ammo(file_data.vcro._ammo),
      engine_type(file_data.engine_type), aggressiveness(file_data.aggressiveness)
{
    setExperienceLevel(file_data.vcro.experience_level);
    setOwner(file_data.vcro.owner);
    setId(file_data.vcro.id);
    setDamage(file_data.vcro.damage);
    setShield(file_data.shield);
    setName(getBasicString(file_data.vcro.name));
    setFCode(getBasicString(file_data.fcode));
    setFlags(file_data.flags);
}

GSimShip::~GSimShip()
{ }

int
GSimShip::getCrew() const
{
    return crew;
}

void
GSimShip::setCrew(int crew)
{
    this->crew = crew;
    setChanged();
}

int
GSimShip::getHull() const
{
    return hull;
}

/** Set hull type. If this changes the hull type, it will also update
    the arms and crew to reflect that new type. */
void
GSimShip::setHull(int hull)
{
    if (hull != this->hull) {
        this->hull = hull;
        if (hull) {
            /* it's a real hull. Fill in defaults. */
            GHull& def = ::getHull(hull);

            // beams
            this->beam_count = def.getMaxBeams();
            if (this->beam_count) {
                if (!this->beam_type)
                    this->beam_type = NUM_BEAMS;
            }

            // torps
            if (def.getNumBays()) {
                this->torp_lcount = this->torp_type = 0;
                this->bay_count = def.getNumBays();
                this->ammo = def.getCargo();
            } else if (def.getMaxLaunchers()) {
                this->torp_lcount = def.getMaxLaunchers();
                if (!this->torp_type)
                    this->torp_type = NUM_TORPS;
                this->ammo = def.getCargo();
                this->bay_count = 0;
            } else {
                this->torp_lcount = this->torp_type = 0;
                this->bay_count = 0;
                this->ammo = 0;
            }

            // rest
            this->crew = def.getCrew();
            this->mass = def.getMass();
        }
        setChanged();
    }
}

int
GSimShip::getMass() const
{
    return mass;
}

void
GSimShip::setMass(int mass)
{
    this->mass = mass;
    setChanged();
}

int
GSimShip::getBeamType() const
{
    return beam_type;
}

void
GSimShip::setBeamType(int beam_type)
{
    this->beam_type = beam_type;
    setChanged();
}

int
GSimShip::getBeamCount() const
{
    return beam_count;
}

void
GSimShip::setBeamCount(int beam_count)
{
    this->beam_count = beam_count;
    setChanged();
}

int
GSimShip::getTorpType() const
{
    return torp_type;
}

void
GSimShip::setTorpType(int torp_type)
{
    this->torp_type = torp_type;
    setChanged();
}

int
GSimShip::getTorpLauncherCount() const
{
    return torp_lcount;
}

void
GSimShip::setTorpLauncherCount(int torp_lcount)
{
    this->torp_lcount = torp_lcount;
    setChanged();
}

int
GSimShip::getBayCount() const
{
    return bay_count;
}

void
GSimShip::setBayCount(int bay_count)
{
    this->bay_count = bay_count;
    setChanged();
}

int
GSimShip::getAmmo() const
{
    return ammo;
}

void
GSimShip::setAmmo(int ammo)
{
    this->ammo = ammo;
    setChanged();
}

int
GSimShip::getEngineType() const
{
    return engine_type;
}

void
GSimShip::setEngineType(int engine_type)
{
    this->engine_type = engine_type;
    setChanged();
}

int
GSimShip::getAggressiveness() const
{
    return aggressiveness;
}

void
GSimShip::setAggressiveness(int aggressiveness)
{
    this->aggressiveness = aggressiveness;
    setChanged();
}

GSimShip*
GSimShip::asShip()
{
    return this;
}

/******************************* GSimPlanet ******************************/

GSimPlanet::GSimPlanet()
    : defense(10), base_defense(10), beam_tech(0),
      torp_tech(1)
{
    for (int i = 0; i < NUM_TORPS; ++i)
        base_torps[i] = 0;
}

GSimPlanet::GSimPlanet(const TSimPlanetData& file_data, int version)
    : defense(file_data.defense),
      base_defense(file_data.base_defense),
      beam_tech(file_data.beam_tech),
      torp_tech(file_data.torp_tech)
{
    // FIXME
    for (int i = 0; i < NUM_TORPS; ++i)
        base_torps[i] = 0;
    if (beam_tech < 0 || beam_tech > NUM_BEAMS)
        beam_tech = 0;
    if (torp_tech <= 0 || torp_tech > NUM_TORPS)
        torp_tech = 1;
    if (base_defense < 0 || base_defense > 200 /* FIXME */)
        base_defense = 0;
    if (defense < 0 || defense > 1000 /* FIXME */)
        defense = 0;

    // setExperienceLevel(file_data.experience_level); // FIXME: struct is wrong
    setOwner(file_data.owner);
    setId(file_data.id);
    setDamage(0); // FIXME?
    setShield(100); // FIXME?
    setName("Planet"); /* FIXME: use spec */
    setFCode(getBasicString(file_data.fcode));
    setFlags(file_data.flags);
}
      
GSimPlanet::~GSimPlanet()
{ }

int
GSimPlanet::getDefense() const
{
    return defense;
}

void
GSimPlanet::setDefense(int defense)
{
    this->defense = defense;
    setChanged();
}

int
GSimPlanet::getBaseDefense() const
{
    return base_defense;
}

void
GSimPlanet::setBaseDefense(int base_defense)
{
    this->base_defense = base_defense;
    setChanged();
}

int
GSimPlanet::getBaseBeamTech() const
{
    return beam_tech;
}

void
GSimPlanet::setBaseBeamTech(int beam_tech)
{
    this->beam_tech = beam_tech;
    setChanged();
}

int
GSimPlanet::getBaseTorpTech() const
{
    return torp_tech;
}

void
GSimPlanet::setBaseTorpTech(int torp_tech)
{
    this->torp_tech = torp_tech;
    setChanged();
}

int
GSimPlanet::getBaseFighters() const
{
    return base_fighters;
}

void
GSimPlanet::setBaseFighters(int base_fighters)
{
    this->base_fighters = base_fighters;
    setChanged();
}

int
GSimPlanet::getBaseTorps(int tech) const
{
    ASSERT(tech > 0 && tech <= NUM_TORPS);
    return base_torps[tech-1];
}

void
GSimPlanet::setBaseTorps(int tech, int amount)
{
    ASSERT(tech > 0 && tech <= NUM_TORPS);
    this->base_torps[tech-1] = amount;
    setChanged();
}

GSimPlanet*
GSimPlanet::asPlanet()
{
    return this;
}
