/**
  *  \file game/sim.h
  */
#ifndef PCC_V2_GAME_SIM_H
#define PCC_V2_GAME_SIM_H

#include <vector>
#include "cpluslib/object.h"
#include "game/spec.h"

class GSimShip;
class GSimPlanet;

class GSimObject : public Object {
    // common
    int  id;
    char name[20];
    char fcode[3];
    int  damage;
    int  shield;
    int  owner;
    int  experience_level;
    int  aggressiveness;
    int  flags;

 protected:
    void setChanged();
 public:
    GSimObject();
    ~GSimObject();

    int  getId() const;
    void setId(int id);

    string_t getName() const;
    void     setName(string_t name);

    string_t getFCode() const;
    void     setFCode(string_t fcode);

    int  getDamage() const;
    void setDamage(int damage);

    int  getShield() const;
    void setShield(int shield);

    int  getOwner() const;
    void setOwner(int owner);

    int  getExperienceLevel() const;
    void setExperienceLevel(int experience_level);

    int  getFlags() const;
    void setFlags(int flags);

    virtual GSimShip*   asShip();
    virtual GSimPlanet* asPlanet();

    /** Flag bits. */
    enum {
        fl_RandomFC  = 1,
        fl_RandomFC1 = 2,
        fl_RandomFC2 = 4,
        fl_RandomFC4 = 8
    };
};

class GSimShip : public GSimObject {
    // ships
    int  crew;
    int  hull;
    int  mass;                  /* custom ships */
    int  beam_type, beam_count;
    int  torp_type, torp_lcount;
    int  bay_count;
    int  ammo;
    int  engine_type;
    int  aggressiveness;

 public:
    GSimShip();
    GSimShip(const TSimShipData& file_data, int version);
    ~GSimShip();

    int  getCrew() const;
    void setCrew(int crew);

    int  getHull() const;
    void setHull(int hull);

    int  getMass() const;
    void setMass(int mass);

    int  getBeamType() const;
    void setBeamType(int beam_type);
    int  getBeamCount() const;
    void setBeamCount(int beam_count);

    int  getTorpType() const;
    void setTorpType(int torp_type);
    int  getTorpLauncherCount() const;
    void setTorpLauncherCount(int torp_lcount);

    int  getBayCount() const;
    void setBayCount(int bay_count);

    int  getAmmo() const;
    void setAmmo(int ammo);

    int  getEngineType() const;
    void setEngineType(int engine_type);
    
    int  getAggressiveness() const;
    void setAggressiveness(int aggressiveness);

    GSimShip* asShip();

    /** Aggressiveness values. Values between 1 and 12 correspond to primary
        enemy settings. */
    enum {
        /// "Kill" mission.
        agg_Kill    = -1,
        /// Passive unit.
        agg_Passive = 0,
        /// Unit has no fuel.
        agg_NoFuel  = 13
    };

};

class GSimPlanet : public GSimObject {
    // planets
    int  defense;
    int  base_defense;
    int  beam_tech;             /* 0 = no base */
    int  torp_tech;
    int  base_fighters;
    int  base_torps[NUM_TORPS];

 public:
    GSimPlanet();
    GSimPlanet(const TSimPlanetData& file_data, int version);
    ~GSimPlanet();

    int  getDefense() const;
    void setDefense(int defense);

    int  getBaseDefense() const;
    void setBaseDefense(int base_defense);

    int  getBaseBeamTech() const;
    void setBaseBeamTech(int beam_tech);

    int  getBaseTorpTech() const;
    void setBaseTorpTech(int torp_tech);

    int  getBaseFighters() const;
    void setBaseFighters(int base_fighters);

    int  getBaseTorps(int tech) const;
    void setBaseTorps(int tech, int amount);

    GSimPlanet* asPlanet();
};

class GSimState {
    std::vector<GSimShip> ships;
    GSimPlanet            planet;
    bool                  has_planet;
 public:
    GSimState();
    GSimState(const GSimState& other);
    ~GSimState();

    GSimState& operator=(const GSimState& other);

    bool hasPlanet() const
        { return has_planet; }
    void addPlanet();
    void removePlanet();
};

#endif
