/*
 *  TEST: Script compilation
 *
 *  This takes a script on stdin and compiles it.
 */

#include <iostream>
#include "int/csource.h"
#include "int/stmt.h"
#include "int/bytecode.h"
#include "io/streams.h"
#include "int/data.h"
#include "int/exec.h"
#include "util/console.h"

class StdinStream : public Stream {
 public:
    std::size_t read(char* buffer, std::size_t max) {
        std::cin.read(buffer, max);
        return std::cin.gcount();
    }
    std::size_t write(const char* /*buffer*/, std::size_t /*max*/) { return 0; }
    int getCapabilities() const { return C_READ; }
    bool seek(fpos_t /*pos*/) { return false; }
    fpos_t getPos() { return -1; }
    fpos_t getSize() { return -1; }

    string_t getName() { return "<stdin>"; }
};

class TestContext : public IntContext {
    static IntObject values[26];
 public:
    handle_t  lookup(string_t name);
    IntObject read(handle_t);
    void      store(handle_t, IntObject);
    bool next() { return false; }
    handle_t  lookupSub(string_t name) { return -1; }
    void      call(handle_t, int, IntExecutionContext&) { }
    TestContext* clone() { return new TestContext(*this); }
};

IntObject TestContext::values[26];

TestContext::handle_t
TestContext::lookup(string_t name)
{
    if (name.length() == 1 && name[0] >= 'A' && name[0] <= 'Z')
        return name[0] - 'A';
    else
        return -1;
}

IntObject
TestContext::read(handle_t n)
{
    return values[n];
}

void
TestContext::store(handle_t n, IntObject v)
{
    values[n] = v;
    std::cout << "setting " << char('A' + n) << " to "
              << (v.convertToString() ? v.getStringVal() : string_t("strange value")) << "\n";
}

int
main(int argc, char** argv)
{
    IntByteCodeObject& bco = IntByteCodeObject::allocBCO();
    IntStreamCommandSource cs(new StdinStream());
    IntStatementCompiler com(cs, bco, 0);

    IntExecutionContext init;
    init.pushContext(new TestContext());

    enableStdioConsole();
    try {
        while (com.parseWholeStatement(true))
            ;

        std::cout << "-- Bytecode follows: --\n";
        bco.dump(std::cout);

        while (argc-- > 1) {
            std::cout << "Running it...\n";
            IntExecutionContext ex(init);
            ex.pushFrame(new IntFrame(bco, ex.getControlStack().size()));
            ex.run();
        }
    }
    catch (IntException& e) {
        std::cerr << "error: " << e.what() << "\n";
    }
}
